/***************************************************************************
 *   Copyright 2007 by Marco Martin <notmart@gmail.com>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "trash.h"

#include <QGraphicsSceneDragDropEvent>

#include <KDebug>
#include <KLocale>
#include <KIconLoader>
#include <KMimeType>
#include <KPropertiesDialog>
#include <KRun>
#include <KSharedConfig>
#include <KMessageBox>
#include <KUrl>


#include <konq_operations.h>
#include <plasma/layouts/boxlayout.h>
#include <plasma/widgets/icon.h>
#include <plasma/containment.h>

Trash::Trash(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_icon(0),
      m_trashUrl(KUrl("trash:/"))
{
    KConfigGroup cg = config();
    int size = globalConfig().readEntry("IconSize", IconSize(KIconLoader::Desktop));
    size = cg.readEntry("IconSize", size);

    createMenu();

    new Plasma::HBoxLayout(this);
    m_icon = new Plasma::Icon(KIcon("user-trash"),QString(),this);
    connect(m_icon, SIGNAL(pressed(bool, QGraphicsSceneMouseEvent*)), this, SLOT(popup(bool,QGraphicsSceneMouseEvent*)));
    m_icon->setIconSize(size, size);
    //icon->setFlag(ItemIsMovable, false);
    resize(m_icon->sizeHint());

    setAcceptDrops(true);
}

Trash::~Trash()
{
}

void Trash::saveState(KConfigGroup *cg)
{
    cg->writeEntry("IconSize", m_icon->iconSize().toSize());
}


void Trash::createMenu()
{
    QAction* open = new QAction(SmallIcon("document-open"),i18n("&Open"), this);
    actions.append(open);
    connect(open, SIGNAL(triggered(bool)), this , SLOT(slotOpen()));

    QAction* empty = new QAction(SmallIcon("trash-empty"),i18n("&Empty trashcan"), this);
    actions.append(empty);
    connect(empty, SIGNAL(triggered(bool)), this , SLOT(slotEmpty()));

    m_menu.addAction(open);
    m_menu.addAction(empty);
}

void Trash::popup(bool pressed, QGraphicsSceneMouseEvent *event)
{
    if (!pressed) {
        return;
    }

    if (!m_menu.isVisible()) {
        QPointF scenePos = mapToScene(boundingRect().topLeft());
        QWidget *viewWidget = event->widget() ? event->widget()->parentWidget() : 0;
        QGraphicsView *view = qobject_cast<QGraphicsView*>(viewWidget);
        if (view) {
            QPoint viewPos = view->mapFromScene(scenePos);
            QPoint globalPos = view->mapToGlobal(viewPos);
            globalPos.ry() -= m_menu.sizeHint().height();
            m_menu.popup(globalPos);
        }
    }

}

void Trash::slotOpen()
{
    if (containment()) {
        containment()->emitLaunchActivated();
    }
    KRun::runUrl(m_trashUrl, "inode/directory", 0);
}

void Trash::slotEmpty()
{
    if (containment()) {
        containment()->emitLaunchActivated();
    }
    const QString text(i18nc("@info", "Do you really want to empty the Trash? All items will get deleted."));
    const bool del = KMessageBox::warningContinueCancel(&m_menu,
                                                        text,
                                                        QString(),
                                                        KGuiItem(i18nc("@action:button", "Empty Trash"),
                                                                  KIcon("user-trash"))
                                                        ) == KMessageBox::Continue;
    if (del) {
        KonqOperations::emptyTrash(&m_menu);
    }
}

QList<QAction*> Trash::contextActions()
{
  return actions;
}

/*void Trash::propertiesDialog()
{
    if (m_dialog == 0) {
        m_dialog = new KPropertiesDialog(m_url);
        connect(m_dialog, SIGNAL(applied()), this, SLOT(acceptedPropertiesDialog()));
    }

    m_dialog->show();
}*/

void Trash::dropEvent(QGraphicsSceneDragDropEvent *event)
{
    if (KUrl::List::canDecode(event->mimeData())) {
        KUrl::List urls = KUrl::List::fromMimeData(event->mimeData());

        if (urls.count() > 0) {
            event->accept();
            KonqOperations::copy( &m_menu, KonqOperations::MOVE, urls, m_trashUrl );
        }
    }

}

#include "trash.moc"

